@@ -14,7 +14,13 @@ class DaikinAirBase(DaikinBRP069):
     TRANSLATIONS = dict(
         DaikinBRP069.TRANSLATIONS,
         **{
-            'mode': {'0': 'fan', '1': 'hot', '2': 'cool', '3': 'auto', '7': 'dry',},
+            'mode': {
+                '0': 'fan',
+                '1': 'hot',
+                '2': 'cool',
+                '3': 'auto',
+                '7': 'dry',
+            },
             'f_rate': {
                 '1': 'low',
                 '3': 'mid',
@@ -45,7 +51,8 @@ class DaikinAirBase(DaikinBRP069):
         return response
 
     def __init__(self, device_id, session=None):
-        """Init the pydaikin appliance, representing one Daikin AirBase (BRP15B61) device."""
+        """Init the pydaikin appliance, representing one Daikin AirBase
+        (BRP15B61) device."""
         super().__init__(device_id, session)
         self.values.update({'htemp': '-', 'otemp': '-', 'shum': '--'})
 
@@ -91,12 +98,12 @@ class DaikinAirBase(DaikinBRP069):
         """Set settings on Daikin device."""
         await self._update_settings(settings)
 
-        self.values['f_auto'] = '1' if 'a' in self.values["f_rate"] else '0'
+        self.values.setdefault('f_airside', 0)
         query_c = (
             'aircon/set_control_info'
             '?pow={pow}&mode={mode}&stemp={stemp}&shum={shum}'
             '&f_rate={f_rate[0]}&f_auto={f_auto}&f_dir={f_dir}'
-            '&lpw=&f_airside=0'
+            '&lpw=&f_airside={f_airside}'
         ).format(**self.values)
 
         _LOGGER.debug("Sending query_c: %s", query_c)
@@ -131,7 +138,8 @@ class DaikinAirBase(DaikinBRP069):
         self.values['zone_onoff'] = quote(';'.join(zone_onoff)).lower()
 
         query = 'aircon/set_zone_setting?zone_name={}&zone_onoff={}'.format(
-            current_state['zone_name'], self.values['zone_onoff'],
+            current_state['zone_name'],
+            self.values['zone_onoff'],
         )
 
         _LOGGER.debug("Set zone:: %s", query)
